from docassemble.base.util import DAList, DAObject

__all__ = ['FishList', 'Fish']


class FishList(DAList):

    def init(self, *pargs, **kwargs):
        self.object_type = Fish
        self.complete_attribute = 'fish_complete'
        super().init(*pargs, **kwargs)


class Fish(DAObject):

    @property
    def fish_complete(self):
        self.common_name  # pylint: disable=pointless-statement
        self.scales  # pylint: disable=pointless-statement
        self.species  # pylint: disable=pointless-statement

    def __str__(self):
        return self.common_name



from docassemble.base.util import store_variables_snapshot, DAObject, user_info, start_time, variables_snapshot_connection

__all__ = ['MyIndex']


class MyIndex(DAObject):

    def init(self, *pargs, **kwargs):
        super().init(*pargs, **kwargs)
        if not hasattr(self, 'data'):
            self.data = {}
        if not hasattr(self, 'key'):
            self.key = 'myindex'

    def save(self):
        data = dict(self.data)
        data['session'] = user_info().session
        data['filename'] = user_info().filename
        data['start_time'] = start_time().astimezone()
        store_variables_snapshot(data, key=self.key)

    def set(self, data):
        if not isinstance(data, dict):
            raise Exception("MyIndex.set: data parameter must be a dictionary")
        self.data = data
        self.save()

    def update(self, new_data):
        self.data.update(new_data)
        self.save()

    def report(self, filter_by=None, order_by=None):
        if filter_by is None:
            filter_string = ''
        else:
            filter_string = ' and ' + filter_by
        if order_by is None:
            order_string = ''
        else:
            order_string = ' order by ' + order_by
        conn = variables_snapshot_connection()
        with conn.cursor() as cur:
            cur.execute("select data from jsonstorage where tags='" + self.key + "'" + filter_string + order_string)
            results = [record[0] for record in cur.fetchall()]
        conn.close()
        return results



# do not pre-load
import docassemble.base.functions


def my_name_suffix():
    return ['Jr', 'Sr', 'II', 'III', 'IV', 'Esq', 'PhD']

docassemble.base.functions.update_language_function('en', 'name_suffix', my_name_suffix)



# do not pre-load
from flask import request, jsonify
from flask_cors import cross_origin
from docassemble.webapp.app_object import app, csrf
from docassemble.webapp.server import api_verify, jsonify_with_status
from docassemble.webapp.worker_common import workerapp
from docassemble.base.config import in_celery
if not in_celery:
    from docassemble.demo.custombg import custom_add_four, custom_comma_and_list


@app.route('/api/start_process', methods=['GET'])
@csrf.exempt
@cross_origin(origins='*', methods=['GET', 'HEAD'], automatic_options=True)
def start_process():
    if not api_verify():
        return jsonify_with_status({"success": False, "error_message": "Access denied."}, 403)
    try:
        operand = int(request.args['operand'])
    except:
        return jsonify_with_status({"success": False, "error_message": "Missing or invalid operand."}, 400)
    task = custom_add_four.delay(operand)
    return jsonify({"success": True, 'task_id': task.id})


@app.route('/api/poll_for_result', methods=['GET'])
@csrf.exempt
@cross_origin(origins='*', methods=['GET', 'HEAD'], automatic_options=True)
def poll_for_result():
    if not api_verify():
        return jsonify_with_status({"success": False, "error_message": "Access denied."}, 403)
    try:
        result = workerapp.AsyncResult(id=request.args['task_id'])
    except:
        return jsonify_with_status({"success": False, "error_message": "Invalid task_id."}, 400)
    if not result.ready():
        return jsonify({"success": True, "ready": False})
    if result.failed():
        return jsonify({"success": False, "ready": True})
    return jsonify({"success": True, "ready": True, "answer": result.get()})


@app.route('/api/start_process_2', methods=['GET'])
@csrf.exempt
@cross_origin(origins='*', methods=['GET', 'HEAD'], automatic_options=True)
def start_process_2():
    if not api_verify():
        return jsonify_with_status({"success": False, "error_message": "Access denied."}, 403)
    task = custom_comma_and_list.delay('foo', 'bar', 'foobar')
    return jsonify({"success": True, 'task_id': task.id})



from docassemble.base.util import as_datetime, DAFile
from ics import Calendar, Event, Attendee, Organizer

__all__ = ['make_event']


def make_event(title=None, location=None, description=None, begin_date=None, begin_time=None, end_date=None, end_time=None, organizer=None, attendees=None):
    if attendees is None:
        attendees = []
    if title is None:
        raise Exception("make_event: a title parameter is required")
    if begin_date is None:
        raise Exception("make_event: a begin_date parameter is required")
    if begin_time is None:
        raise Exception("make_event: a begin_time parameter is required")
    if end_date is None:
        raise Exception("make_event: an end_date parameter is required")
    if end_time is None:
        raise Exception("make_event: an end_time parameter is required")
    c = Calendar()
    e = Event()
    if organizer is not None:
        e.organizer = Organizer(common_name=organizer.name.full(), email=organizer.email)
    if len(attendees) > 0:
        e.attendees = [Attendee(common_name=attendee.name.full(), email=attendee.email) for attendee in attendees]
    e.name = str(title)
    e.begin = as_datetime(begin_date.replace_time(begin_time), timezone='UTC').format_datetime('yyyy-MM-dd HH:mm:ss')
    e.end = as_datetime(end_date.replace_time(end_time), timezone='UTC').format_datetime('yyyy-MM-dd HH:mm:ss')
    if location not in (None, ''):
        e.location = str(location)
    if description not in (None, ''):
        e.description = str(description)
    c.events.add(e)
    c.events  # pylint: disable=pointless-statement
    ics_file = DAFile('ics_file')
    ics_file.set_random_instance_name()
    ics_file.initialize(filename="event.ics", mimetype="text/calendar")
    with open(ics_file.path(), 'w', encoding='utf-8') as f:
        f.write(str(c))
    ics_file.commit()
    return ics_file



import re
import subprocess

__all__ = ['language_list', 'lang_name', 'get_fonts']


def language_list():
    return [('ab', 'Abkhazian'), ('aa', 'Afar'), ('af', 'Afrikaans'), ('ak', 'Akan'), ('sq', 'Albanian'), ('am', 'Amharic'), ('ar', 'Arabic'), ('an', 'Aragonese'), ('hy', 'Armenian'), ('as', 'Assamese'), ('av', 'Avaric'), ('ae', 'Avestan'), ('ay', 'Aymara'), ('az', 'Azerbaijani'), ('bm', 'Bambara'), ('ba', 'Bashkir'), ('eu', 'Basque'), ('be', 'Belarusian'), ('bn', 'Bengali'), ('bh', 'Bihari languages'), ('bi', 'Bislama'), ('bs', 'Bosnian'), ('br', 'Breton'), ('bg', 'Bulgarian'), ('my', 'Burmese'), ('ca', 'Catalan'), ('ch', 'Chamorro'), ('ce', 'Chechen'), ('zh', 'Chinese'), ('cu', 'Church Slavic'), ('cv', 'Chuvash'), ('kw', 'Cornish'), ('co', 'Corsican'), ('cr', 'Cree'), ('hr', 'Croatian'), ('cs', 'Czech'), ('da', 'Danish'), ('dv', 'Dhivehi'), ('nl', 'Dutch'), ('dz', 'Dzongkha'), ('en', 'English'), ('eo', 'Esperanto'), ('et', 'Estonian'), ('ee', 'Ewe'), ('fo', 'Faroese'), ('fj', 'Fijian'), ('fi', 'Finnish'), ('fr', 'French'), ('ff', 'Fulah'), ('gl', 'Galician'), ('lg', 'Ganda'), ('ka', 'Georgian'), ('de', 'German'), ('gn', 'Guarani'), ('gu', 'Gujarati'), ('ht', 'Haitian'), ('ha', 'Hausa'), ('he', 'Hebrew'), ('hz', 'Herero'), ('hi', 'Hindi'), ('ho', 'Hiri Motu'), ('hu', 'Hungarian'), ('is', 'Icelandic'), ('io', 'Ido'), ('ig', 'Igbo'), ('id', 'Indonesian'), ('ia', 'Interlingua (International Auxiliary Language Association)'), ('ie', 'Interlingue'), ('iu', 'Inuktitut'), ('ik', 'Inupiaq'), ('ga', 'Irish'), ('it', 'Italian'), ('ja', 'Japanese'), ('jv', 'Javanese'), ('kl', 'Kalaallisut'), ('kn', 'Kannada'), ('kr', 'Kanuri'), ('ks', 'Kashmiri'), ('kk', 'Kazakh'), ('km', 'Khmer'), ('ki', 'Kikuyu'), ('rw', 'Kinyarwanda'), ('ky', 'Kirghiz'), ('kv', 'Komi'), ('kg', 'Kongo'), ('ko', 'Korean'), ('kj', 'Kuanyama'), ('ku', 'Kurdish'), ('lo', 'Lao'), ('la', 'Latin'), ('lv', 'Latvian'), ('li', 'Limburgan'), ('ln', 'Lingala'), ('lt', 'Lithuanian'), ('lu', 'Luba-Katanga'), ('lb', 'Luxembourgish'), ('mk', 'Macedonian'), ('mg', 'Malagasy'), ('ms', 'Malay (macrolanguage)'), ('ml', 'Malayalam'), ('mt', 'Maltese'), ('gv', 'Manx'), ('mi', 'Maori'), ('mr', 'Marathi'), ('mh', 'Marshallese'), ('el', 'Modern Greek (1453-)'), ('mn', 'Mongolian'), ('na', 'Nauru'), ('nv', 'Navajo'), ('ng', 'Ndonga'), ('ne', 'Nepali (macrolanguage)'), ('nd', 'North Ndebele'), ('se', 'Northern Sami'), ('no', 'Norwegian'), ('nb', 'Norwegian Bokmål'), ('nn', 'Norwegian Nynorsk'), ('ny', 'Nyanja'), ('oc', 'Occitan (post 1500)'), ('oj', 'Ojibwa'), ('or', 'Oriya (macrolanguage)'), ('om', 'Oromo'), ('os', 'Ossetian'), ('pi', 'Pali'), ('pa', 'Panjabi'), ('fa', 'Persian'), ('pl', 'Polish'), ('pt', 'Portuguese'), ('ps', 'Pushto'), ('qu', 'Quechua'), ('ro', 'Romanian'), ('rm', 'Romansh'), ('rn', 'Rundi'), ('ru', 'Russian'), ('sm', 'Samoan'), ('sg', 'Sango'), ('sa', 'Sanskrit'), ('sc', 'Sardinian'), ('gd', 'Scottish Gaelic'), ('sr', 'Serbian'), ('sh', 'Serbo-Croatian'), ('sn', 'Shona'), ('ii', 'Sichuan Yi'), ('sd', 'Sindhi'), ('si', 'Sinhala'), ('sk', 'Slovak'), ('sl', 'Slovenian'), ('so', 'Somali'), ('nr', 'South Ndebele'), ('st', 'Southern Sotho'), ('es', 'Spanish'), ('su', 'Sundanese'), ('sw', 'Swahili (macrolanguage)'), ('ss', 'Swati'), ('sv', 'Swedish'), ('tl', 'Tagalog'), ('ty', 'Tahitian'), ('tg', 'Tajik'), ('ta', 'Tamil'), ('tt', 'Tatar'), ('te', 'Telugu'), ('th', 'Thai'), ('bo', 'Tibetan'), ('ti', 'Tigrinya'), ('to', 'Tonga (Tonga Islands)'), ('ts', 'Tsonga'), ('tn', 'Tswana'), ('tr', 'Turkish'), ('tk', 'Turkmen'), ('tw', 'Twi'), ('ug', 'Uighur'), ('uk', 'Ukrainian'), ('ur', 'Urdu'), ('uz', 'Uzbek'), ('ve', 'Venda'), ('vi', 'Vietnamese'), ('vo', 'Volapük'), ('wa', 'Walloon'), ('cy', 'Welsh'), ('fy', 'Western Frisian'), ('wo', 'Wolof'), ('xh', 'Xhosa'), ('yi', 'Yiddish'), ('yo', 'Yoruba'), ('za', 'Zhuang'), ('zu', 'Zulu')]


def lang_name(language):
    for item in language_list():
        if language == item[0]:
            return item[1]
    return language


def get_fonts(language):
    arguments = ['fc-list', ':lang=' + language, 'family', 'file']
    try:
        completed_process = subprocess.run(arguments, timeout=600, check=False, capture_output=True)
        result = completed_process.returncode
        if result != 0:
            error_message = completed_process.stderr.decode()
    except subprocess.TimeoutExpired:
        result = 1
        error_message = "call to fc-list timed out"
    if result == 0:
        return sorted([x.split(': ') for x in re.sub(',', ', ', completed_process.stdout.decode()).split("\n") if x != ''], key=lambda y: y[1])
    return [error_message]



from docassemble.base.util import DAGoogleAPI, DAFile
import apiclient

api = DAGoogleAPI()

__all__ = ['get_folder_names', 'get_files_in_folder', 'write_file_to_folder', 'download_file']


def get_folder_names():
    service = api.drive_service()
    items = []
    while True:
        response = service.files().list(spaces="drive", fields="nextPageToken, files(id, name)", q="mimeType='application/vnd.google-apps.folder' and sharedWithMe").execute()
        for the_file in response.get('files', []):
            items.append(the_file)
        page_token = response.get('nextPageToken', None)
        if page_token is None:
            break
    return [item['name'] for item in items]


def get_folder_id(folder_name):
    service = api.drive_service()
    response = service.files().list(spaces="drive", fields="nextPageToken, files(id, name)", q="mimeType='application/vnd.google-apps.folder' and sharedWithMe and name='" + str(folder_name) + "'").execute()
    folder_id = None
    for item in response.get('files', []):
        folder_id = item['id']
    return folder_id


def get_file_id(filename, folder_name):
    folder_id = get_folder_id(folder_name)
    if folder_id is None:
        raise Exception("The folder was not found")
    service = api.drive_service()
    file_id = None
    response = service.files().list(spaces="drive", fields="nextPageToken, files(id, name)", q="mimeType!='application/vnd.google-apps.folder' and '" + str(folder_id) + "' in parents and name='" + str(filename) + "'").execute()
    for item in response.get('files', []):
        file_id = item['id']
    return file_id


def get_files_in_folder(folder_name):
    folder_id = get_folder_id(folder_name)
    if folder_id is None:
        raise Exception("The folder was not found")
    service = api.drive_service()
    items = []
    while True:
        response = service.files().list(spaces="drive", fields="nextPageToken, files(id, name)", q="mimeType!='application/vnd.google-apps.folder' and trashed=false and '" + str(folder_id) + "' in parents").execute()
        for the_file in response.get('files', []):
            items.append(the_file)
        page_token = response.get('nextPageToken', None)
        if page_token is None:
            break
    return [item['name'] for item in items]


def write_file_to_folder(path, mimetype, filename, folder_name):
    folder_id = get_folder_id(folder_name)
    if folder_id is None:
        raise Exception("The folder was not found")
    service = api.drive_service()
    file_metadata = {'name': filename, 'parents': [folder_id]}
    media = apiclient.http.MediaFileUpload(path, mimetype=mimetype)
    the_new_file = service.files().create(body=file_metadata,
                                          media_body=media,
                                          fields='id').execute()
    return the_new_file.get('id')


def download_file(filename, folder_name):
    file_id = get_file_id(filename, folder_name)
    if file_id is None:
        raise Exception("The file was not found")
    the_file = DAFile()
    the_file.set_random_instance_name()
    the_file.initialize(filename=filename)
    service = api.drive_service()
    with open(the_file.path(), 'wb') as fh:
        response = service.files().get_media(fileId=file_id)
        downloader = apiclient.http.MediaIoBaseDownload(fh, response)
        done = False
        while done is False:
            status, done = downloader.next_chunk()  # pylint: disable=unused-variable
    the_file.commit()
    return the_file



# do not pre-load
from docassemble.webapp.worker_common import workerapp, bg_context, worker_controller as wc


@workerapp.task
def custom_add_four(operand):
    return operand + 4


@workerapp.task
def custom_comma_and_list(*pargs):
    with bg_context():
        return wc.util.comma_and_list(*pargs)



__all__ = ['make_pie']

import matplotlib
import matplotlib.pyplot as plt

matplotlib.use('svg')

def make_pie(data, the_file):
    the_file.initialize(filename='graph.svg')
    fig = plt.figure(figsize=(6, 6))
    ax = fig.add_axes([0.1, 0.1, 0.8, 0.8])

    labels = []
    fracs = []
    for key, val in data.items():
        labels.append(key)
        fracs.append(val)

    ax.pie(fracs, labels=labels, autopct='%1.1f%%')
    with open(the_file.path(), 'wb') as f:
        plt.savefig(f, format="svg")
    the_file.commit()
    the_file.retrieve()



import re

__all__ = ['prog_disclose']


def prog_disclose(template, classname=None):
    if classname is None:
        classname = ' bg-secondary-subtle'
    else:
        classname = ' ' + classname.strip()
    the_id = re.sub(r'[^A-Za-z0-9]', '', template.instanceName)
    return """\
<a class="collapsed" data-bs-toggle="collapse" href="#{the_id}" role="button" aria-expanded="false" aria-controls="collapseExample"><span class="pdcaretopen"><i class="fa-solid fa-caret-down"></i></span><span class="pdcaretclosed"><i class="fa-solid fa-caret-right"></i></span> {the_subject}</a>
<div class="collapse" id="{the_id}"><div class="card card-body{the_classname} pb-1">{the_content}</div></div>\
""".format(the_id=the_id, the_subject=template.subject_as_html(trim=True), the_classname=classname, the_content=template.content_as_html())



from docassemble.base.util import Thing, prevent_dependency_satisfaction

__all__ = ['Legume']


class Legume(Thing):

    @prevent_dependency_satisfaction
    def is_tasty(self):
        return bool(self.sweet_index > 5 or self.savory_index > 6)



import string
import random
from docassemble.base.util import interview_list, create_user, get_user_list, get_user_info, set_user_info, manage_privileges

__all__ = ['get_permissions']

r = random.SystemRandom()

other_user = 29
other_interview = 'docassemble.base:data/questions/examples/madlibs.yml'


def random_alphanumeric(length):
    return ''.join(r.choice(string.ascii_uppercase + string.ascii_lowercase + string.digits) for x in range(length))


def random_lower_string(length):
    return ''.join(r.choice(string.ascii_lowercase) for i in range(length))


def get_permissions():
    results = {}
    try:
        result = interview_list(user_id='all', filename=other_interview)
        assert result is not None
        result = interview_list(user_id=other_user, filename=other_interview)
        assert result is not None
        results['access_sessions'] = True
    except Exception as err:
        results['access_sessions'] = err.__class__.__name__ + ": " + str(err)
    try:
        result = interview_list(user_id=other_user, action='delete_all', filename=other_interview)
        assert result is not None
        result = interview_list(user_id='all', action='delete_all', filename=other_interview)
        assert result is not None
        results['edit_sessions'] = True
    except Exception as err:
        results['edit_sessions'] = err.__class__.__name__ + ": " + str(err)
    try:
        get_user_info(user_id=other_user)
        user_list = []
        next_id = None
        while True:
            (items, next_id) = get_user_list(next_id=next_id)
            user_list.extend(items)
            if not next_id:
                break
        assert len(user_list) > 0
        results['access_user_info'] = True
    except Exception as err:
        results['access_user_info'] = err.__class__.__name__ + ": " + str(err)
    try:
        create_user(random_lower_string(8) + '@docassemble.org', random_alphanumeric(8), privileges=['user', 'customer'], info={'first_name': 'John', 'last_name': 'Smith'})
        results['create_user'] = True
    except Exception as err:
        results['create_user'] = err.__class__.__name__ + ": " + str(err)
    try:
        set_user_info(user_id=other_user, first_name='Foo', last_name='Bar')
        results['edit_user_info'] = True
    except Exception as err:
        results['edit_user_info'] = err.__class__.__name__ + ": " + str(err)
    try:
        set_user_info(user_id=other_user, password=random_lower_string(8))
        results['edit_user_password'] = True
    except Exception as err:
        results['edit_user_password'] = err.__class__.__name__ + ": " + str(err)
    try:
        set_user_info(user_id=other_user, active=False)
        set_user_info(user_id=other_user, active=True)
        results['edit_user_active_status'] = True
    except Exception as err:
        results['edit_user_active_status'] = err.__class__.__name__ + ": " + str(err)
    try:
        set_user_info(user_id=other_user, privileges=['user', 'trainer', 'customer'])
        set_user_info(user_id=other_user, privileges=['user', 'trainer'])
        results['edit_user_privileges'] = True
    except Exception as err:
        results['edit_user_privileges'] = err.__class__.__name__ + ": " + str(err)
    try:
        result = manage_privileges('list')
        assert result is not None
        result = manage_privileges('inspect', 'customer')
        assert result is not None
        results['access_privileges'] = True
    except Exception as err:
        results['access_privileges'] = err.__class__.__name__ + ": " + str(err)
    test_privilege = random_lower_string(10)
    try:
        result = manage_privileges('add', test_privilege)
        assert result is not None
        result = manage_privileges('remove', test_privilege)
        assert result is not None
        results['edit_privileges'] = True
    except Exception as err:
        results['edit_privileges'] = err.__class__.__name__ + ": " + str(err)
    return results



import pandas
from docassemble.base.util import path_and_mimetype

__all__ = ['get_fruit_names', 'fruit_info']

fruit_info_by_name = {}
fruit_names = []


def read_data(filename):
    the_xlsx_file, mimetype = path_and_mimetype(filename)  # pylint: disable=unused-variable
    df = pandas.read_excel(the_xlsx_file)
    for indexno in df.index:
        if not df['Name'][indexno]:
            continue
        fruit_names.append(df['Name'][indexno])
        fruit_info_by_name[df['Name'][indexno]] = {"color": df['Color'][indexno], "seeds": df['Seeds'][indexno]}


def get_fruit_names():
    return fruit_names


def fruit_info(fruit):
    if fruit not in fruit_info_by_name:
        raise Exception("Reference to invalid fruit " + fruit)
    return fruit_info_by_name[fruit]

read_data('docassemble.demo:data/sources/fruit_data.xlsx')



# do not pre-load
import random
import re
import string

__all__ = ['start_accordion', 'next_accordion', 'end_accordion']

r = random.SystemRandom()


def _get_section_id(section):
    section_id = re.sub(r'[^a-z0-9]+', '-', section.lower())
    if len(section_id) > 0:
        section_id += '-'
    section_id += (''.join(r.choice(string.ascii_lowercase) for i in range(5)))
    return section_id


def _header(section_id):
    return f"""\
<div class="row mb-3">
  <div class="col">
    <div class="accordion" id="{section_id}">
    """


def _section_start(section_id, section, showing):
    if showing:
        show = ' show'
        expanded = 'true'
        collapsed = ''
    else:
        show = ''
        expanded = 'false'
        collapsed = ' collapsed'
    return f"""\
      <div class="accordion-item">
        <h2 class="accordion-header">
          <button class="accordion-button{collapsed}" type="button" data-bs-toggle="collapse" data-bs-target="#collapse-{section_id}" aria-expanded="{expanded}" aria-controls="collapse-{section_id}">
            {section}
          </button>
        </h2>
        <div id="collapse-{section_id}" class="accordion-collapse collapse{show}" data-bs-parent="#{section_id}">
          <div class="accordion-body">
"""


def _section_end():
    return """\
          </div>
        </div>
      </div>
"""


def _footer():
    return """\
    </div>
  </div>
</div>
    """


def start_accordion(section, showing=False):
    """Returns HTML for the start of a series of accordion
    sections. The end_accordion() function must be called at some
    point later in the page, or else the HTML will be corrupted. If
    you want the section to be open when the page loads, set
    showing=True.

    """
    section_id = _get_section_id(section)
    return _header(section_id) + _section_start(section_id, section, showing)


def next_accordion(section, showing=False):
    """Returns HTML for ending a previous according section and
    starting a new accordion section. Can only be used after
    start_accordion(). If you want the section to be open when the
    page loads, set showing=True.

    """
    section_id = _get_section_id(section)
    return _section_end() + _section_start(section_id, section, showing)


def end_accordion():
    """Returns HTML for ending a series of accordion sections. Can
    only be used after next_accordion or start_accordion().

    """
    return _section_end() + _footer()



from docassemble.base.util import DADict

__all__ = ['IncomeDict']


class IncomeDict(DADict):

    def hook_on_gather(self, *pargs, **kwargs):
        if 'benefits' in self.elements and self['benefits'].receives and 'employment' in self.elements and self['employment'].receives and self['benefits'].amount + self['employment'].amount > 2000:
            self.reason_for_benefits  # pylint: disable=pointless-statement
        elif hasattr(self, 'reason_for_benefits'):
            del self.reason_for_benefits

    def hook_after_gather(self, *pargs, **kwargs):
        self.total_amount = sum(y.amount for y in self.values() if y.receives)



import io
import docassemble.base.util
from google.cloud import vision

__all__ = ['gv_ocr']


def gv_ocr(the_file):
    api = docassemble.base.util.DAGoogleAPI()
    client = api.google_cloud_vision_client()
    image = vision.Image()
    with io.open(the_file.path(), 'rb') as image_file:
        content = image_file.read()
    image = vision.Image(content=content)
    response = client.text_detection(image=image)
    output = ''
    for text in response.text_annotations:
        output += text.description + "\n"
    if response.error.message:
        raise Exception(response.error.message)
    return output

# def async_detect_document(gcs_source_uri, gcs_destination_uri):
#     mime_type = 'application/pdf'

#     # How many pages should be grouped into each json output file.
#     batch_size = 2

#     client = api.google_cloud_vision_client()

#     feature = vision.Feature(
#         type_=vision.Feature.Type.DOCUMENT_TEXT_DETECTION)

#     gcs_source = vision.GcsSource(uri=gcs_source_uri)
#     input_config = vision.InputConfig(
#         gcs_source=gcs_source, mime_type=mime_type)

#     gcs_destination = vision.GcsDestination(uri=gcs_destination_uri)
#     output_config = vision.OutputConfig(
#         gcs_destination=gcs_destination, batch_size=batch_size)

#     async_request = vision.AsyncAnnotateFileRequest(
#         features=[feature], input_config=input_config,
#         output_config=output_config)

#     operation = client.async_batch_annotate_files(
#         requests=[async_request])

#     print('Waiting for the operation to finish.')
#     operation.result(timeout=420)

#     # Once the request has completed and the output has been
#     # written to GCS, we can list all the output files.
#     storage_client = storage.Client()

#     match = re.match(r'gs://([^/]+)/(.+)', gcs_destination_uri)
#     bucket_name = match.group(1)
#     prefix = match.group(2)

#     bucket = storage_client.get_bucket(bucket_name)

#     # List objects with the given prefix, filtering out folders.
#     blob_list = [blob for blob in list(bucket.list_blobs(
#         prefix=prefix)) if not blob.name.endswith('/')]
#     print('Output files:')
#     for blob in blob_list:
#         print(blob.name)

#     # Process the first output file from GCS.
#     # Since we specified batch_size=2, the first response contains
#     # the first two pages of the input file.
#     output = blob_list[0]

#     json_string = output.download_as_string()
#     response = json.loads(json_string)

#     # The actual response for the first page of the input file.
#     first_page_response = response['responses'][0]
#     annotation = first_page_response['fullTextAnnotation']

#     # Here we print the full text from the first page.
#     # The response contains more information:
#     # annotation/pages/blocks/paragraphs/words/symbols
#     # including confidence scores and bounding boxes
#     print('Full text:\n')
#     print(annotation['text'])



import re

__all__ = ['card_start', 'card_end']


def card_start(label, color=None, icon=None):
    if color not in ('primary',
                     'secondary',
                     'success',
                     'danger',
                     'warning',
                     'info',
                     'light',
                     'dark',
                     'link'):
        color_text = ''
    else:
        color_text = ' text-bg-' + color
    if icon is None:
        icon_text = ''
    else:
        icon_text = re.sub(r'^(fa[a-z])-fa-', r'\1 fa-', str(icon))
        if not re.search(r'^fa[a-z] fa-', icon_text):
            icon_text = 'fa-solid fa-' + icon
        icon_text = re.sub(r'^fas ', 'fa-solid ', icon_text)
        icon_text = re.sub(r'^far ', 'fa-regular ', icon_text)
        icon_text = re.sub(r'^fab ', 'fa-brands ', icon_text)
        icon_text = '<i class="' + icon_text + '"></i> '
    return f'<div class="card{color_text} mb-3" markdown="span"><div class="card-body" markdown="1"><h2 class="card-title h4" markdown="span">{icon_text}{label}</h2>'


def card_end():
    return '</div></div>'



# do not pre-load
import unittest
from docassemble.base.util import path_and_mimetype, url_of


class TestNameSuffix(unittest.TestCase):

    def setUp(self):
        pass

    def test_path(self):
        (path, mimetype) = path_and_mimetype('lithuania.svg')
        self.assertTrue(path.endswith('site-packages/docassemble/demo/data/static/lithuania.svg'))
        self.assertEqual(mimetype, 'image/svg+xml')

    def test_url(self):
        self.assertTrue(url_of('lithuania.svg').startswith('/packagestatic/docassemble.demo/lithuania.svg'))

if __name__ == '__main__':
    from docassemble.webapp.server import TestContext
    with TestContext('docassemble.demo'):
        unittest.main()



# do not pre-load
import unittest
from docassemble.demo.change_suffix import my_name_suffix


class TestNameSuffix(unittest.TestCase):

    def setUp(self):
        pass

    def test_jr(self):
        self.assertIn('Jr', my_name_suffix())

    def test_length(self):
        self.assertEqual(len(my_name_suffix()), 7)

if __name__ == '__main__':
    unittest.main()



from docassemble.base.util import explain

__all__ = ['wrong_vegetable']


def wrong_vegetable(vegetable):
    if vegetable == 'turnip':
        explain("You also said your favorite vegetable was turnip.")
        return True
    explain("You also said your favorite vegetable was " + vegetable + ".")
    return False



# do not pre-load
from docassemble.base.util import current_context, Individual, log

__all__ = ['AltIndividual']


class AltIndividual(Individual):

    def __str__(self):
        if current_context().inside_of != 'standard':
            return self.name.full()
        return self.name.familiar()



from docassemble.base.util import variables_snapshot_connection, user_info

__all__ = ['analyze']


def analyze():
    conn = variables_snapshot_connection()
    cur = conn.cursor()
    cur.execute("select data->>'favorite_fruit' from jsonstorage where filename='" + user_info().filename + "'")
    counts = {}
    for record in cur.fetchall():
        fruit = record[0].lower()
        if fruit not in counts:
            counts[fruit] = 0
        counts[fruit] += 1
    conn.close()
    return counts



import json
import gspread
from docassemble.base.util import get_config
from oauth2client.service_account import ServiceAccountCredentials
credential_json = get_config('google').get('service account credentials', None)
if credential_json is None:
    credential_info = None
else:
    credential_info = json.loads(credential_json, strict=False)

scope = ['https://spreadsheets.google.com/feeds',
         'https://www.googleapis.com/auth/drive']

__all__ = ['read_sheet', 'append_to_sheet']


def read_sheet(sheet_name, worksheet_index=0):
    creds = ServiceAccountCredentials.from_json_keyfile_dict(credential_info, scope)
    client = gspread.authorize(creds)
    sheet = client.open(sheet_name).get_worksheet(worksheet_index)
    return sheet.get_all_records()


def append_to_sheet(sheet_name, vals, worksheet_index=0):
    creds = ServiceAccountCredentials.from_json_keyfile_dict(credential_info, scope)
    client = gspread.authorize(creds)
    sheet = client.open(sheet_name).get_worksheet(worksheet_index)
    sheet.append_row(vals)



from docassemble.base.util import DAGoogleAPI
import apiclient

__all__ = ['fetch_file']


def fetch_file(file_id, path):
    service = DAGoogleAPI().drive_service()
    with open(path, 'wb') as fh:
        response = service.files().get_media(fileId=file_id)
        downloader = apiclient.http.MediaIoBaseDownload(fh, response)
        done = False
        while done is False:
            status, done = downloader.next_chunk()  # pylint: disable=unused-variable



import re
from docassemble.base.util import log, get_config
from googleapiclient.discovery import build

__all__ = ['translate_phrase']


def translate_phrase(phrase, source_language, target_language):
    try:
        api_key = get_config('google')['api key']
    except:
        log("Could not translate because Google API key was not in Configuration.")
        return phrase
    try:
        service = build('translate', 'v2',
                        developerKey=api_key)
        resp = service.translations().list(
            source=source_language,
            target=target_language,
            q=[phrase]
        ).execute()
        return re.sub(r'&#39;', r"'", str(resp['translations'][0]['translatedText']))
    except Exception as err:
        log("translation failed: " + err.__class__.__name__ + ": " + str(err))
        return phrase
